import subprocess

import sys
import yaml


class ApiError(Exception):
    pass


def get(kind, selector=()):
    selector_query = sum(
        (['-l', '{}={}'.format(key, value)] for key, value in selector),
        [])
    get_process = subprocess.Popen(
        ['kubectl', 'get', kind, *selector_query, '-o', 'yaml'],
        stdout=subprocess.PIPE, stderr=sys.stderr)
    objects = yaml.load(get_process.stdout)
    if get_process.wait() != 0:
        raise ApiError
    return objects


def create(definition):
    run_command_with_yaml_on_stdin(
        ['kubectl', 'create', '-f', '-'], definition)


def apply(definition):
    run_command_with_yaml_on_stdin(
        ['kubectl', 'apply', '--record', '-f', '-'], definition)


def replace(definition):
    run_command_with_yaml_on_stdin(
        ['kubectl', 'replace', '--force', '--cascade', '--record', '-f', '-'], definition)


def rolling_update(definition, name):
    run_command_with_yaml_on_stdin(
        ['kubectl', 'rolling-update', name, '-f', '-'],
        definition)


def delete(kind, name):
    subprocess.check_call(['kubectl', 'delete', kind, name])


def run_command_with_yaml_on_stdin(command, definition):
    create_process = subprocess.Popen(command, stdin=subprocess.PIPE, stderr=sys.stderr)
    create_process.communicate(yaml.dump(definition).encode())
    create_process.stdin.close()
    if create_process.wait() != 0:
        raise ApiError()



#!/usr/bin/env python

import optparse
import pathlib

from kubepy import appliers
from kubepy import base_commands


class InstallError(Exception):
    pass


class ApplyOneCommand(base_commands.BaseCommand):
    def get_optparser(self):
        parser = optparse.OptionParser(
            usage="usage: %prog [options] [job_name ...]",
            epilog="Installs or Executes selected definition"
        )
        parser.add_option(
            '--directory', dest='directory', default='.',
            help='installs definitions from this directory')
        base_commands.add_container_options(parser)
        return parser

    def handle(self, args, options):
        directory = pathlib.Path(options.directory).resolve()
        runner = appliers.DirectoryApplier(directory, options)
        if args:
            for job_name in args:
                runner.apply_named(job_name)
        else:
            raise base_commands.CommandError('Provide definition names.')


def run():
    ApplyOneCommand().run()

if __name__ == '__main__':
    run()



import copy


def tag_untaged_images(definition, tag):
    new_definition = copy.deepcopy(definition)
    for container in iterate_container_definitions(new_definition):
        container['image'] = tag_untaged_image(container['image'], tag)
    return new_definition


def iterate_container_definitions(definition):
    new_spec = definition['spec']
    new_template = new_spec['template']
    yield from new_template['spec']['containers']


def tag_untaged_image(image, tag):
    if ':' in image:
        return image
    else:
        return image + ':' + tag



#!/usr/bin/env python
import optparse
import pathlib

from kubepy import appliers
from kubepy import base_commands


class InstallAllCommand(base_commands.BaseCommand):
    def handle(self, args, options):
        directory = pathlib.Path(options.directory).resolve()
        applier = appliers.DirectoryApplier(directory, options)
        applier.apply_all()

    def get_optparser(self):
        parser = optparse.OptionParser(
            usage="usage: %prog [options] [name ...]",
            epilog="Apply definitions from given directory."
        )
        parser.add_option(
            '--directory', dest='directory', default='.',
            help='installs all definitions from this directory')
        base_commands.add_container_options(parser)
        return parser


def run():
    InstallAllCommand().run()


if __name__ == '__main__':
    run()



import copy

import time

from kubepy import api
from kubepy import definition_manager
from kubepy import definition_transformers


class InstallError(Exception):
    pass


class JobError(InstallError):
    pass


class DeadlineExceeded(JobError):
    pass


class DirectoryApplier:
    def __init__(self, path, options):
        self.options = options
        self.manager = definition_manager.DefinitionManager(path)

    def apply_all(self):
        for definition in self.manager.get_sorted_definitions():
            UniversalDefinitionApplier(definition, self.options).apply()

    def apply_named(self, name):
        definition = self.manager.get_definition(name)
        UniversalDefinitionApplier(definition, self.options).apply()


class BaseDefinitionApplier:
    def __init__(self, definition, options):
        self.definition = definition
        self.options = options

    def apply(self):
        raise NotImplementedError

    @property
    def usable_with(self):
        raise NotImplementedError


class ResourceApplier(BaseDefinitionApplier):
    usable_with = ['Service', 'Secret']

    def apply(self):
        api.apply(self.definition)


class DeploymentApplier(BaseDefinitionApplier):
    usable_with = ['Deployment']

    def apply(self):
        if self.options.replace:
            api.replace(self.new_definition)
        else:
            api.apply(self.new_definition)

    @property
    def new_definition(self):
        return transform_container_definition(self.definition, self.options)


class JobApplier(BaseDefinitionApplier):
    usable_with = ['Job']

    def apply(self):
        api.create(self.new_definition)
        try:
            while True:
                status = api.get('job', [('app', self.app)])['items'][0]['status']
                if 'completionTime' in status:
                    break
                elif 'conditions' in status:
                    condition = status['conditions'][0]
                    if condition['type'] == 'Failed':
                        self._handle_failed_condition(condition)
                else:
                    time.sleep(1)
        finally:
            api.delete('job', self.name)

    def _handle_failed_condition(self, condition):
        if condition['reason'] == 'DeadlineExceeded':
            raise DeadlineExceeded(condition['message'])
        else:
            raise JobError(condition['message'])

    @property
    def new_definition(self):
        return transform_container_definition(self.definition, self.options)

    @property
    def name(self):
        return self.definition['metadata']['name']

    @property
    def app(self):
        return self.definition['metadata']['labels']['app']


class UniversalDefinitionApplier(BaseDefinitionApplier):
    applier_classes = (ResourceApplier, DeploymentApplier, JobApplier)
    usable_with = sum((applier.usable_with for applier in applier_classes), [])

    def apply(self):
        kind = self.definition['kind']
        try:
            applier_class = self.kind_map[kind]
        except KeyError:
            raise InstallError('Unknown resource kind: {}'.format(kind))
        else:
            applier = applier_class(self.definition, self.options)
            applier.apply()

    @property
    def kind_map(self):
        kind_map = UniqueDict()
        for applier_class in self.applier_classes:
            for kind in applier_class.usable_with:
                kind_map[kind] = applier_class
        return dict(kind_map)


class UniqueDictException(Exception):
    pass


class UniqueDict(dict):
    def __setitem__(self, key, value):
        if key not in self:
            super().__setitem__(key, value)
        else:
            raise UniqueDictException(key)


def transform_container_definition(definition, options):
    new_definition = copy.deepcopy(definition)
    new_definition = definition_transformers.tag_untaged_images(new_definition, options.build_tag)
    return new_definition



import pathlib

import yaml


class DefinitionManager:
    def __init__(self, directory: pathlib.Path):
        self.directory = directory

    def get_definition(self, name):
        path = self.directory / (name + '.yml')
        return self.get_yaml_from_file(path)

    def get_sorted_definitions(self):
        for path in sorted(self.directory.iterdir()):
            yield self.get_yaml_from_file(path)

    def get_yaml_from_file(self, path):
        with path.open() as yaml_file:
            return yaml.load(yaml_file)



import sys

from kubepy import definition_transformers


class CommandError(Exception):
    def __init__(self, message):
        super().__init__(message)
        self.message = message


class BaseCommand:
    def run(self):
        parser = self.get_optparser()
        (options, names) = parser.parse_args()
        try:
            self.handle(names, options)
        except CommandError as e:
            print(e.message)
            parser.print_usage()
            sys.exit(1)

    def handle(self, args, options):
        raise NotImplementedError

    def get_optparser(self):
        raise NotImplementedError


def add_container_options(parser):
    parser.add_option(
        '--build-tag', dest='build_tag', action='store', default='latest',
        help='used image tag and name suffix')
    parser.add_option(
        '--replace', dest='replace', action='store_true', default=False,
        help='delete and recreate deployments instead of doing rolling update')
